#!/usr/bin/python3

import logging
from logging.handlers import RotatingFileHandler
import os
import gzip
import shutil

#定义日志的格式 
FORMAT = "%(asctime)s [%(levelname)s]: %(message)s"
LOGDIR = "/var/log/RevisionsManager/"

class DuplicateFilter(logging.Filter):
    def __init__(self, threshold):
        super().__init__()
        self.last_logs = []
        self.repeat_count = {}
        self.threshold = threshold

    def filter(self, record):
        current_log = record.msg.split(",")[0]
        if current_log not in self.last_logs:
            self.last_logs.append(current_log)
            self.repeat_count[current_log] = 1
            if len(self.last_logs) > 5:
                oldest_log = self.last_logs.pop(0)
                del self.repeat_count[oldest_log]
        else:
            self.repeat_count[current_log] += 1

        return self.repeat_count[current_log] <= self.threshold

class CompressingRotatingFileHandler(RotatingFileHandler):
    """
    自定义的 RotatingFileHandler,增加日志压缩功能,并检测 .gz 文件
    """
    # 覆盖父类的 doRollover 方法，按照压缩文件的逻辑轮换日志
    def doRollover(self):
        
        if self.stream:
            self.stream.close()
            self.stream = None

        # 轮换和压缩日志文件时，检测压缩后的文件名
        for i in range(self.backupCount - 1, 0, -1):
            sfn = f"{self.baseFilename}.{i}.gz"
            dfn = f"{self.baseFilename}.{i + 1}.gz"
            if os.path.exists(sfn):
                if os.path.exists(dfn):
                    os.remove(dfn)
                os.rename(sfn, dfn)

        # 压缩当前日志文件
        dfn = f"{self.baseFilename}.1.gz"
        if os.path.exists(dfn):
            os.remove(dfn)
        self.compress_log(self.baseFilename, dfn)

        # 重新打开新的日志文件进行写入
        if not self.delay:
            self.stream = self._open()

    def compress_log(self, log_filename, gz_filename):
        """
        压缩日志文件为 .gz 格式
        """
        with open(log_filename, 'rb') as f_in:
            with gzip.open(gz_filename, 'wb') as f_out:
                shutil.copyfileobj(f_in, f_out)
        os.remove(log_filename)

class RMLog():
    def __init__(self) -> None:
        loglevel = logging.DEBUG
        logFileName = LOGDIR + "RevisionsManager.log"
        logging.basicConfig(format=FORMAT, level=loglevel, datefmt='%m-%d,%H:%M:%S',
                            handlers=[CompressingRotatingFileHandler(logFileName, maxBytes=10 * 1024, backupCount=10)])
        logging.getLogger().addFilter(DuplicateFilter(threshold=20))
        logging.info("RMLog init file[%s]", logFileName)




'''
#定义日志的格式 
FORMAT = "%(asctime)s [%(levelname)s]: %(message)s"
LOGDIR = "/var/log/RevisionsManager/"


def get_fileSize(filePath):
    fsize = os.path.getsize(filePath)
    fsize_M = fsize / float(1024 * 1024)
    return round(fsize_M, 2)

def get_logfile():
    numlist = []
    if not os.path.exists(LOGDIR):
        os.makedirs(LOGDIR)
    #优先获取当前未写满100M的日志编号文件
    for i in os.listdir(LOGDIR):
        if "RevisionsManager.log." in os.path.basename(LOGDIR + i):
            numlist.append((LOGDIR + i).split(".")[2])
            if get_fileSize(LOGDIR + i) < 10:
                return LOGDIR + i

    #获取1-5未使用的最小数字的标号作为日志文件
    for i in range(1, 6):
        if str(i) not in numlist:
            return(os.path.join(LOGDIR, ("RevisionsManager.log.%s") % i))
    try:
        #编号1-5日志文件均写满时，删除第一个，往前移动日志编号，获取最后一个编号作为日志文件
        if len(numlist) != 0:
            os.remove(os.path.join(LOGDIR, "RevisionsManager.log.1"))
            for i in range(2, 6):
                os.rename(os.path.join(LOGDIR, ("RevisionsManager.log.%s") % i),
                        os.path.join(LOGDIR, ("RevisionsManager.log.%s") % (i - 1)))
            return os.path.join(LOGDIR, "RevisionsManager.log.5")
        #默认情况下未生成任何日志时使用编号1的日志文件
        else:
            return os.path.join(LOGDIR, "RevisionsManager.log.1")
    except:
        return os.path.join(LOGDIR, "RevisionsManager.log.1")

class RMLog():
    def __init__(self) -> None:

        # loglevel = logging.INFO
        loglevel = logging.DEBUG
        logFileName = get_logfile()
        logging.basicConfig(format=FORMAT, level=loglevel, datefmt='%m-%d,%H:%M:%S', filename=logFileName, filemode='a')
        logging.info("RMLog init file[%s]", logFileName)

'''
